@@ -19,6 +19,7 @@
 from pathlib import Path
 import re
 import os
+import json
 
 from sequana.lazy import pandas as pd
 from sequana.lazy import pylab
@@ -26,6 +27,7 @@ from sequana.lazy import numpy as np
 from sequana import logger
 from matplotlib_venn import venn2_unweighted, venn3_unweighted
 from sequana.rnadiff import RNADiffResults
+from sequana.summary import Summary
 
 try:
     import gseapy
@@ -36,17 +38,17 @@ except:
 __all__ = ["PantherEnrichment", "KeggPathwayEnrichment"]
 
 
-class PantherEnrichment():  
+class PantherEnrichment():
     """
 
-        # This will read your rnadiff results and tstore the rlevants genes into 
+        # This will read your rnadiff results and tstore the rlevants genes into
         # mygenes_u, mygenes_down, mygenes attributes.
 
         By default, we keep only the genes with a adjusted pvalue <= 0.05. The
         fold_change threshold is on a log2 scale and set to 0 (meaning no
         filtering). Only one value is requested and
         used to filter out positive and negative fold change (on the log2
-        scale). In other word, a log2 fold change threshold of 2 means that we 
+        scale). In other word, a log2 fold change threshold of 2 means that we
         filter out values between -2 and 2.
 
         If you prefer to work in natural scale, just set the parameter
@@ -82,6 +84,11 @@ class PantherEnrichment():
                 show_pvalues=True, fdr_threshold=0.05)
 
 
+        The number of genes is limited to about 3100 depending (don't ask me
+        why, this seem to be a hard-coded limitation on PantherDB website).
+        In such case, you should add a filter e.g on padj or fold change
+
+
     """
     def __init__(self, folder, requests_per_sec=10, padj_threshold=0.05,
         log2_fc_threshold=0, fc_threshold=None,
@@ -92,15 +99,12 @@ class PantherEnrichment():
         if fc_threshold is not None:
             log2_fc_threshold = pylab.log2(fc_threshold)
 
-        from bioservices import panther, quickgo, uniprot
+        from bioservices import panther, quickgo
         self.panther = panther.Panther()
         self.valid_taxons = [x['taxon_id'] for x in self.panther.get_supported_genomes()]
 
         self.quickgo = quickgo.QuickGO(cache=True)
-        self.uniprot = uniprot.UniProt(cache=True)
-
         self.quickgo.requests_per_sec = requests_per_sec
-        self.uniprot.requests_per_sec = requests_per_sec
 
         self.ancestors = {"MF": "GO:0003674", "CC": "GO:0005575", "BP": "GO:0008150"}
         self.aspects = {"MF": "molecular_function"}
@@ -134,6 +138,16 @@ class PantherEnrichment():
         self.mygenes = list(self.mygenes.sort_values('padj').index)
         self.mygenes_down = list(self.mygenes_down.sort_values('padj').index)
         self.mygenes_up = list(self.mygenes_up.sort_values('padj').index)
+        logger.info("Found {} genes ({} up; {} down)".format(
+            len(self.mygenes),
+            len(self.mygenes_down),
+            len(self.mygenes_up)))
+        self.summary = {
+            "DGE": {
+                "down": len(self.mygenes_down),
+                "up": len(self.mygenes_up)}
+
+            }
 
         # When using ENSEMBL, prefix "gene:"  should be removed to be understood
         # by PantherDB
@@ -141,13 +155,20 @@ class PantherEnrichment():
         self.mygenes_down = [x.replace("gene:", "") for x in self.mygenes_down]
         self.mygenes_up = [x.replace("gene:", "") for x in self.mygenes_up]
 
-        logger.info("Kept {} genes ({} up; {} down)".format(
-            len(self.mygenes),
-            len(self.mygenes_down),
-            len(self.mygenes_up)))
+        if fc_threshold!=0 and padj_threshold!=0.05:
+            logger.info("Kept {} genes ({} up; {} down)".format(
+                len(self.mygenes),
+                len(self.mygenes_down),
+                len(self.mygenes_up)))
 
-    def compute_enrichment(self, mygenes, taxid, ontologies=None, 
-            enrichment_test="FISHER", correction="FDR"):
+        self.summary["DGE_after_filtering"] = {
+            "up": len(self.mygenes_up),
+            "down": len(self.mygenes_down)
+        }
+
+
+    def compute_enrichment(self, mygenes, taxid, ontologies=None,
+            enrichment_test="FISHER", correction="FDR", progress=True):
         #taxid=83333 # ecoli
         """
         :param enrichment_test: Fisher or Binomial
@@ -155,15 +176,15 @@ class PantherEnrichment():
 
         The field **number_in_reference** indicates from the reference, the number
         of genes that have a given ontolgy term. For instance, 998 genes have
-        the term. This is stored in **number_in_reference**. If the reference 
+        the term. This is stored in **number_in_reference**. If the reference
         contains 4391 genes, and you provided 49
         genes , the **expected** number of genes that have this ontology term is
-        49*998/4391 that is 11.1369, which is stored in **"expected**. 
+        49*998/4391 that is 11.1369, which is stored in **"expected**.
 
         Now, if you actually find that 14 out of 49
         genes have the term, you need to compare the numbers 11.1369 and 14. Are
         they really different ? The ratio 14 / 11.1369 is stored in
-        **fold_enrichment**. The pvalue and FDR are stored as well. 
+        **fold_enrichment**. The pvalue and FDR are stored as well.
 
 
         Some genes may be missing If you provide 50 genes, you may end up with
@@ -240,6 +261,9 @@ class PantherEnrichment():
 
     def _get_name_given_accession(self, accession):  #pragma: no cover
         from bioservices import UniProt
+        self.uniprot = uniprot.UniProt(cache=True)
+        self.uniprot.requests_per_sec = 10
+
         acc = [x for x in accession.split("|") if x.startswith("UniProtKB")]
         acc = acc[0].split("=")[1]
         res = self.uniprot.get_df(acc, limit=1)
@@ -258,21 +282,21 @@ class PantherEnrichment():
         goids = defaultdict(list)
         for x in info:
             name = x['persistent_id']
-            if "annotation_type_list" in x.keys(): 
-                data = x["annotation_type_list"]["annotation_data_type"] 
-                for annot in data: 
+            if "annotation_type_list" in x.keys():
+                data = x["annotation_type_list"]["annotation_data_type"]
+                for annot in data:
                     if 'content' not in annot:
                         print(x, annot)
                     print(annot['content'])
-                    if annot["content"] in annotation_list: 
-                        if "annotation" in annot['annotation_list']: 
-                            subdata = annot["annotation_list"]["annotation"] 
- 
-                            if isinstance(subdata,dict): 
-                                goids[name].append(subdata["id"]) 
-                            else: 
-                                for item in subdata: 
-                                    goids[name].append(item["id"]) 
+                    if annot["content"] in annotation_list:
+                        if "annotation" in annot['annotation_list']:
+                            subdata = annot["annotation_list"]["annotation"]
+
+                            if isinstance(subdata,dict):
+                                goids[name].append(subdata["id"])
+                            else:
+                                for item in subdata:
+                                    goids[name].append(item["id"])
         return goids
 
     def __get_names(self, mapping_results): #pragma: no cover
@@ -298,7 +322,7 @@ class PantherEnrichment():
     def plot_piechart(self, df):
         # Here we show the GO terms that have number in list > 0
         # Note, that this is dangerous to look only at this picture without
-        # the reference plot, which data is not available thourg the pathner API 
+        # the reference plot, which data is not available thourg the pathner API
         labels = []
         for this in df.query("number_in_list!=0").label.values:
             if len(this)>50:
@@ -351,7 +375,7 @@ class PantherEnrichment():
         df["abs_log2_fold_enrichment"] = abs(pylab.log2(df['fold_enrichment']))
 
         # Some user may want to include GO terms with fold enrichment
-        # significanyly below 1 or not. 
+        # significanyly below 1 or not.
         if include_negative_enrichment is False:
             df = df.query("fold_enrichment>=1").copy()
             logger.info("Found {} GO terms after keeping only positive enrichment".format(len(df)))
@@ -362,20 +386,19 @@ class PantherEnrichment():
 
         return df
 
-
     def plot_go_terms(self, ontologies, max_features=50,
         log=False,
         fontsize=8, minimum_genes=0, pvalue=0.05,
         cmap="summer_r",
-        sort_by="fold_enrichment", 
+        sort_by="fold_enrichment",
         show_pvalues=False,
-        include_negative_enrichment=False, 
-        fdr_threshold=0.05, compute_levels=True):
+        include_negative_enrichment=False,
+        fdr_threshold=0.05, compute_levels=True, progress=True):
 
         assert sort_by in ['pValue', 'fold_enrichment', 'fdr']
 
         # FIXME: pvalue and fold_enrichment not sorted in same order
-        pylab.clf()
+        pylab.figure(figsize=(10,6))
 
         df = self.get_data(ontologies,
                 include_negative_enrichment=include_negative_enrichment,
@@ -413,7 +436,7 @@ class PantherEnrichment():
         # We get all levels for each go id.
         # They are stored by MF, CC or BP
         if compute_levels:
-            paths = self.get_graph(list(subdf['id'].values))
+            paths = self.get_graph(list(subdf['id'].values), progress=progress)
             levels = []
             keys = list(paths.keys())
             goid_levels = paths[keys[0]]
@@ -438,9 +461,9 @@ class PantherEnrichment():
         if log:
             pylab.scatter(pylab.log2(subdf.fold_enrichment), range(len(subdf)),
                 c=subdf.fdr,
-                s=sizes, 
+                s=sizes,
                 cmap=cmap,
-                alpha=0.8, 
+                alpha=0.8,
                 ec="k",
                 vmin=0, vmax=fdr_threshold, zorder=10)
             #pylab.barh(range(N), pylab.log2(subdf.fold_enrichment), color="r",
@@ -449,20 +472,18 @@ class PantherEnrichment():
             #pylab.axvline(-1, color="gray", ls="--")
         else:
              pylab.scatter(subdf.fold_enrichment, range(len(subdf)),
-                c=subdf.fdr, cmap=cmap, 
-                s=sizes, 
-                ec="k",alpha=.8, 
+                c=subdf.fdr, cmap=cmap,
+                s=sizes,
+                ec="k",alpha=.8,
                 vmin=0, vmax=fdr_threshold, zorder=10)
             #    pylab.barh(range(N), subdf.fold_enrichment, color="r",
             #    label="not significant")
         pylab.grid(zorder=-10)
-        ax2 = pylab.colorbar(shrink=0.5); 
+        ax2 = pylab.colorbar(shrink=0.5);
         ax2.ax.set_ylabel('FDR')
 
-        
-
         labels = [x if len(x)<50 else x[0:47]+"..." for x in list(subdf.label)]
-        ticks = ["{} ({}) {}".format(ID,level, "; " + label.title()) 
+        ticks = ["{} ({}) {}".format(ID,level, "; " + label.title())
             for level, ID, label in zip(subdf['level'], subdf.id, labels)]
 
         pylab.yticks(range(N), ticks, fontsize=fontsize,ha='left')
@@ -479,8 +500,8 @@ class PantherEnrichment():
         fc_max = subdf.fold_enrichment.max(skipna=True)
         fc_min = subdf.fold_enrichment.min(skipna=True)
         # go into log2 space
-        fc_max = pylab.log2(fc_max) 
-        fc_min = pylab.log2(fc_min) 
+        fc_max = pylab.log2(fc_max)
+        fc_min = pylab.log2(fc_min)
         abs_max = max(fc_max, abs(fc_min), 1)
 
         if log:
@@ -532,7 +553,7 @@ class PantherEnrichment():
                 loc='lower right',
                 ncol=1,
                 frameon=True, title="gene-set size",
-                labelspacing=labelspacing, borderpad=borderpad, 
+                labelspacing=labelspacing, borderpad=borderpad,
                  handletextpad=handletextpad,
                 fontsize=8)
         else:
@@ -542,7 +563,7 @@ class PantherEnrichment():
                 loc='lower right',
                 ncol=1,
                 frameon=True, title="gene-set size",
-                labelspacing=labelspacing, borderpad=borderpad, 
+                labelspacing=labelspacing, borderpad=borderpad,
                  handletextpad=handletextpad,
                 fontsize=8)
 
@@ -555,7 +576,7 @@ class PantherEnrichment():
         self.df = df
         return df
 
-    def get_graph(self, go_ids, ontologies=None):
+    def get_graph(self, go_ids, ontologies=None, progress=True):
         # Here we filter the data to keep only the relevant go terms as shown in
         # panther pie chart
         import networkx as nx
@@ -579,7 +600,7 @@ class PantherEnrichment():
         for i, go_id in enumerate(go_ids):
 
             # Some go terms maybe obsolet or renamed. Looking at other functions
-            # may not work simply because the ID has changed. 
+            # may not work simply because the ID has changed.
             info = self.quickgo.get_go_terms(go_id)
             annotations[go_id] = info
 
@@ -610,10 +631,11 @@ class PantherEnrichment():
                 if len(edges["results"])>=1:
                     for path in edges["results"]:
                         for edge in path:
-                            gg.add_edge(edge['child'], edge["parent"]) 
+                            gg.add_edge(edge['child'], edge["parent"])
                 else:
                     print(_id, edges["results"])
-            pb.animate(i+1)
+            if progress is True:
+                pb.animate(i+1)
 
         self.obsolets = obsolets
         self.annotations = annotations
@@ -655,10 +677,11 @@ class PantherEnrichment():
 
 
 class KeggPathwayEnrichment():
-    """DRAFT IN PROGRESS
+    """Kegg Pathways enrichment from DGE results
 
+    DGE = Differentially Gene Expression
 
-    Current input is the output of the rnadiff analysis
+    Current input is the output of the RNADiff analysis
     ::
 
         pe = PathwayEnrichment("rnadiff", "eco")
@@ -671,36 +694,131 @@ class KeggPathwayEnrichment():
         up.to_csv("kegg_pathway_up_regulated.csv")
         down.to_csv("kegg_pathway_down_regulated.csv")
 
+    This class works like a charm for ecoli with GFF that uses gene names.
+    For mus musculus, organism is **mmu** (not **mus*). you will need to have
+    a mapping of the Ensemble ID into Kegg IDs (actually gene name).
+
+    You can perform the conversion using BioServices/BioMart. We have
+    implemented a simple function::
+
+        from sequana.enrichment import IDConversion
+        conv = IDConversion("mmusculus_gene_ensembl")
+        df = conf.querry()
+        conf.save(df)
+
+    You can then import the dataframe, as follows using the mapper argument::
+
+        import pandas as pd
+        df = pd.read_csv("biomart.csv")
+        df = df.rename({"external_gene_name":"name", "ensembl_gene_id": "ensembl"}, 
+                axis=1)
+        df = df.set_index("ensembl", inplace=True)
+
+        KeggPathwayEnrichment("path_to_rnadiff", "mmu", mapper=df)
+
+    More generally, when starting KeggPathwayEnrichment, we read all pathways.
+    This may change with time. So, you can save the pathways::
+
+
+        ke.export_pathways_to_json()
+
+    And read them back::
+
+        ke = KeggPathwayEnrichment("path_to_rnadiff", "mmu", mapper=df,
+            preload_directory="kegg_pathways/mmu")
+
+
+        df = ke.scatterplot(ke.enrichment['down'])
+        tight_layout()
+        savefig("B4052_T1vsT0_KE_scatterplot_down.png")
+        df = ke.scatterplot(ke.enrichment['up'])
+        savefig("B4052_T1vsT0_KE_scatterplot_up.png")
+
+
+
+
     """
-    def __init__(self, folder, organism, alpha=0.05, log2_fc=0):
+    def __init__(self, folder, organism, alpha=0.05, log2_fc=0, progress=True,
+            mapper=None, background=None, preload_directory=None):
+
+
         print("DRAFT in progress")
         from bioservices import KEGG
         self.kegg = KEGG(cache=True)
         self.kegg.organism = organism
+        self.summary = Summary("KeggPathwayEnrichment")
+        self.summary.add_params({
+            "folder":folder,
+            "organism": organism,
+            "alpha": alpha,
+            "log2_fc": log2_fc,
+            "mapper": (True if mapper is not None else False),
+            "background": background})
 
         self.rnadiff = RNADiffResults(folder ,alpha=alpha, log2_fc=log2_fc)
+        # some clean up
+        if "ID" in self.rnadiff.df.columns:
+            self.rnadiff.df['ID'] = [x.replace("gene:", "") for x in self.rnadiff.df['ID']]
+        self.rnadiff.df.index = [x.replace("gene:", "") for x in self.rnadiff.df.index]
+        for key, values in self.rnadiff.gene_lists.items():
+            self.rnadiff.gene_lists[key] = [x.replace("gene:", "") for x in values]
+
+        self.rnadiff.df.index = [x.replace("gene:", "") for x in self.rnadiff.df.index]
+
         choices = list(self.rnadiff.gene_lists.keys())
 
-        self.background = len(self.kegg.list(self.kegg.organism).split("\n"))
+        if background:
+            self.background = background
+        else:
+            self.background = len(self.kegg.list(self.kegg.organism).split("\n"))
         logger.info("Set number of genes to {}".format(self.background))
 
-        self._load_pathways()
-        self.compute_enrichment()
+        self._load_pathways(progress=progress,
+            preload_directory=preload_directory)
 
+        if isinstance(mapper, str):
+            import pandas as pd
+            df = pd.read_csv(mapper)
+            df = df.rename({"external_gene_name":"name", "ensembl_gene_id": "ensembl"}, axis=1)
+            df.set_index("ensembl", inplace=True)
+            self.mapper = df
 
-    def _load_pathways(self):
+        else: # a dataframe already container the correct columns and index
+            self.mapper = mapper
+
+        try:
+            self.compute_enrichment()
+        except Exception as err:
+            print(err)
+            logger.critical("An error occured while computing enrichments. ")
+
+    def _load_pathways(self, progress=True, preload_directory=None):
         # This is just loading all pathways once for all
-        logger.info("loading all pathways from KEGG. may take time the first time")
         self.pathways = {}
-        from easydev import Progress
-        pb = Progress(len(self.kegg.pathwayIds))
-        for i, ID in enumerate(self.kegg.pathwayIds):
-            self.pathways[ID.replace("path:", "")] = self.kegg.parse(self.kegg.get(ID))
-            pb.animate(i+1)
+        if preload_directory:
+            # preload is a directory with all pathways in it
+            import glob
+            pathways = glob.glob(preload_directory + "/*json")
+            for i, name in enumerate(pathways):
+                key = name.strip(".json").split("/")[-1]
+                with open(name, "r") as fin:
+                    data = json.load(fin)
+                    self.pathways[key] = data
+        else:
+            logger.info("loading all pathways from KEGG. may take time the first time")
+            from easydev import Progress
+            pb = Progress(len(self.kegg.pathwayIds))
+            for i, ID in enumerate(self.kegg.pathwayIds):
+                self.pathways[ID.replace("path:", "")] = self.kegg.parse(self.kegg.get(ID))
+                if progress:
+                    pb.animate(i+1)
 
-        # Some cleanup
+        # Some cleanup. Note that if we read the json file, this is different
+        # since already cleanup but this code does no harm
         for ID in self.pathways.keys():
-            name = self.pathways[ID]['NAME'][0]
+            name = self.pathways[ID]['NAME']
+            if isinstance(name, list):
+                name = name[0]
             self.pathways[ID]['NAME'] = name.split(" - ", 1)[0]
 
         # save gene sets
@@ -708,8 +826,16 @@ class KeggPathwayEnrichment():
         for ID in self.pathways.keys():
             res =  self.pathways[ID]
             if "GENE" in res.keys():
-                results = [res['GENE'][g].split(';')[0] for g in res['GENE'].keys()]
-                
+                results = []
+                # some pathways reports genes as a dictionary id:'gene name; description' ('.eg. eco')
+                # others reports genes as a dictionary id:'description'
+                for geneID, description in res['GENE'].items():
+                    if ";" in description:
+                        name = description.split(';')[0]
+                    else:
+                        name = geneID
+                    results.append(name)
+
                 self.gene_sets[ID] = results
             else:
                 print("SKIPPED (no genes) {}: {}".format(ID, res['NAME']))
@@ -736,6 +862,9 @@ class KeggPathwayEnrichment():
     def compute_enrichment(self, background=None):
         if background is None:
             background = self.background
+
+        self.summary.data["missing_genes"] = {}
+        self.summary.data["input_gene_list"] = {}
         self.enrichment = {}
         self.enrichment['up'] = self._enrichr("up", background=background)
         self.enrichment['down'] = self._enrichr("down", background=background)
@@ -751,6 +880,19 @@ class KeggPathwayEnrichment():
         else:
             assert category in ['up', 'down', 'all']
             gene_list = list(self.rnadiff.gene_lists[category])
+            
+        logger.info("Input gene list of {} ids".format(len(gene_list)))
+        self.summary.data['input_gene_list'][category] = len(gene_list)
+        
+
+        if self.mapper is not None:
+            missing = [x for x in gene_list if x not in self.mapper.index]
+            logger.info("Missing genes from mapper dataframe: {}".format(len(missing)))
+            self.summary.data['missing_genes'][category] = ",".join(missing)
+            gene_list = [x for x in gene_list if x in self.mapper.index]
+            identifiers = self.mapper.loc[gene_list]['name'].drop_duplicates().values
+            logger.info("Mapped gene list of {} ids".format(len(identifiers)))
+            gene_list = list(identifiers)
 
         enr = gseapy.enrichr(
             gene_list=gene_list,
@@ -776,6 +918,8 @@ class KeggPathwayEnrichment():
             nmax = len(df)
         df = df.iloc[0:nmax]
         df = df.sort_values("Adjusted P-value", ascending=False)
+        df = df.rename({"Term": "pathway_id"}, axis=1)
+        df = df[df.columns]
         return df
 
     def barplot(self, enrich, cutoff=0.05, nmax=10):
@@ -797,9 +941,9 @@ class KeggPathwayEnrichment():
         df = self._get_final_df(enrich.results, cutoff=cutoff, nmax=nmax)
 
         pylab.clf()
-        pylab.scatter(-pylab.log10(df['Adjusted P-value']), range(len(df)), s=10*df['size'],    
+        pylab.scatter(-pylab.log10(df['Adjusted P-value']), range(len(df)), s=10*df['size'],
             c=df['size'])
- 
+
         pylab.xlabel("Odd ratio")
         pylab.ylabel("Gene sets")
         pylab.yticks(range(len(df)), df.name)
@@ -827,11 +971,12 @@ class KeggPathwayEnrichment():
 
     def _get_summary_pathway(self, pathway_ID):
         genes = self.df_pathways.loc[pathway_ID]['GENE']
-        df_down = self.rnadiff.df.query("padj<=0.05 and log2FoldChange<0")
-        df_up = self.rnadiff.df.query("padj<=0.05 and log2FoldChange>0")
+        df_down = self.rnadiff.df.query("padj<=0.05 and log2FoldChange<0").copy()
+        df_up = self.rnadiff.df.query("padj<=0.05 and log2FoldChange>0").copy()
 
         #f_down = self.rnadiff.dr_gene_lists[self.comparison]
 
+        logger.info("{}".format(pathway_ID))
         logger.info("Total down-regulated: {}".format(len(df_down)))
         logger.info("Total up-regulated: {}".format(len(df_up)))
 
@@ -839,7 +984,6 @@ class KeggPathwayEnrichment():
         for k,v in genes.items():
             mapper[v.split(";")[0]] = k
         self.genes = genes
-        self.mapper = mapper
         self.df_down = df_down
         self.df_up = df_up
         summary_names = []
@@ -847,19 +991,47 @@ class KeggPathwayEnrichment():
         summary_types = []
         summary_pvalues = []
         summary_fcs = []
+
+        if self.mapper is not None:
+            if 'Name' not in df_down.columns:
+                df_down['Name'] = df_down['ID']
+                Names = []
+                for index in df_down.index:
+                    Names.append(self.mapper.loc[index]['name'][0])
+                df_down['Name'] = Names
+
+            if 'Name' not in df_up.columns:
+                df_up['Name'] = df_up['ID']
+                Names = []
+                for index in df_up.index:
+                    Names.append(self.mapper.loc[index]['name'][0])
+                df_up['Name'] = Names
+
+            #
+            identifiers = []
+            new_mapper = {}
+            for name, kegg_id in mapper.items():
+                try:
+                    identifier = self.mapper.query("name == @name")['name'].drop_duplicates().index[0]
+                    identifiers.append(identifier)
+                    new_mapper[identifier] = kegg_id
+                except:
+                    logger.warning("Skipped {}(kegg ID {}). could not find mapping".format(name, kegg_id))
+            mapper = new_mapper
+
         for name, kegg_id in mapper.items():
             summary_names.append(name)
             summary_keggids.append(kegg_id)
 
             if name.lower() in [x.lower() for x in df_down.Name]:
-                pvalue = -pylab.log10(df_down.query("Name==@name").pvalue.values[0])
+                padj = -pylab.log10(df_down.query("Name==@name").padj.values[0])
                 fc = df_down.query("Name==@name").log2FoldChange.values[0]
                 summary_fcs.append(fc)
-                summary_pvalues.append(pvalue)
+                summary_pvalues.append(padj)
                 summary_types.append("-")
             elif name.lower() in [x.lower() for x in df_up.Name]:
-                pvalue = -pylab.log10(df_up.query("Name==@name").pvalue.values[0])
-                summary_pvalues.append(pvalue)
+                padj = -pylab.log10(df_up.query("Name==@name").padj.values[0])
+                summary_pvalues.append(padj)
                 fc = df_up.query("Name==@name").log2FoldChange.values[0]
                 summary_fcs.append(fc)
                 summary_types.append("+")
@@ -871,17 +1043,17 @@ class KeggPathwayEnrichment():
         summary = pd.DataFrame({
                     "type":summary_types,
                     "name": summary_names,
-                    "pvalue": summary_pvalues,
+                    "padj": summary_pvalues,
                     "fc": summary_fcs,
                     "keggid": summary_keggids
                     })
         summary['description'] = [self.pathways[pathway_ID]['GENE'][x] for x in summary.keggid]
-        return summary 
+        return summary
 
     def _get_colors(self, summary):
         colors = {}
         for index, row in summary.iterrows():
-            pvalue = row['pvalue']
+            pvalue = row['padj']
             type_ = row['type']
             kegg_id = row['keggid']
             if type_ == "-":
@@ -941,11 +1113,12 @@ class KeggPathwayEnrichment():
     def save_all_pathways(self): #pragma: no cover
         # This does not do any enrichment. Just save all pathways once for all
         # with useful information
-        for ID in self.kegg.pathwayIds:
+        for ID in self.pathway.keys():
             self.save_pathway(ID)
 
     def save_significant_pathways(self, mode, cutoff=0.05, nmax=20,
-        background=None):#pragma: no cover  
+        background=None, tag="" ):#pragma: no cover
+        """mode should be up, down or all"""
 
         if background is None:
             background = self.background
@@ -966,11 +1139,10 @@ class KeggPathwayEnrichment():
             summaries[ID] = summary
         return summaries
 
-
     def find_pathways_by_gene(self, gene_name, match="exact"):
         """Returns pathways that contain the gene name
 
-        ke.find_pathways_by_gene("ysgA") 
+        ke.find_pathways_by_gene("ysgA")
         """
 
         #First let us find the kegg ID
@@ -1012,23 +1184,92 @@ class KeggPathwayEnrichment():
         return list(set(paths))
 
 
+    def save_project(self, tag):
+
+
+        for category in ["up", "down", "all"]:
+            self.enrichment[category].results.to_csv("{}_KEGG_GSEA_all.csv")
+            self.enrichment[category].results.query("`Adjusted P-value`<0.05").to_csv(
+                "{}_KEGG_GSEA_significant.csv")
+
+        from pylab import savefig
+        for _type in ['down', 'up', 'all']:
+            df = self.barplot(self.enrichment[_type])
+            savefig("{}_KEGG_{}.png".format(tag, _type), dpi=200)
+            df.to_csv("{}_KEGG_{}.csv".format(tag, _type), index=None)
+
+    def export_pathways_to_json(self, outdir="kegg_pathways"):
+        # This is useful to keep an exact track of the pathways that were used.
+        # They can be loaded back. If so, we use kegg service only in
+        # :meth:`find_pathways_by_gene` method and
+
+        outdir = outdir + "/" + self.kegg.organism
+        from easydev import mkdirs
+        mkdirs(outdir)
+        import json
+        for key, data in self.pathways.items():
+            with open(f"{outdir}/{key}.json", "w") as fout:
+                json.dump(data, fout)
 
 
 
+class IDConversion():
+    """
+
+        conv = IDConversion()
+        df = conv.query()
+        df.set_index("ensembl_gene_id")
+
+    """
+    def __init__(self, dataset="mmusculus_gene_ensembl"):
+
+        from bioservices import BioMart
+        self.biomart = BioMart()
+        self.datasets = self.biomart.get_datasets("ENSEMBL_MART_ENSEMBL")
+        self._dataset = None
+        try:
+            self.dataset = dataset
+        except:
+            logger.critical("Invalid dataset. checks datasets attributse")
 
 
-
-
-
-
-
-
-
-
-
-
-
-
-
+    def _set_dataset(self, dataset):
+        if dataset not in self.datasets['name'].values:
+            raise ValueError("Invalid dataset {}. Check the Choose amongst {}".format(dataset, self.datasets))
+        self._dataset = dataset
+        self.attributes = self.biomart.attributes(dataset=dataset)
+        self.filters = self.biomart.filters(dataset=dataset)
+    def _get_dataset(self):
+        return self._dataset
+    dataset=property(_get_dataset, _set_dataset)
+
+    def query(self, attributes=["ensembl_gene_id", "go_id", "entrezgene_id",
+                                "mgi_id", "external_gene_name"]):
+        logger.info("Please wait. This may take a while depending on your connection")
+        self.biomart.new_query()
+        self.biomart.add_dataset_to_xml(self.dataset)
+        for attribute in attributes:
+            self.biomart.add_attribute_to_xml(attribute)
+        xml = self.biomart.get_xml()
+        results = self.biomart.query(xml)
+
+        import pandas as pd
+        import io
+
+        df = pd.read_csv(io.StringIO(results), sep="\t")
+        df.columns=attributes
+        #df = df.set_index('ensembl_gene_id')
+        # name should be the name used by kegg
+        return df
+
+    def save(self, df):
+        """df is the output of :meth:`~query`. This function save it keeping
+track of day/month/year and dataset."""
+        import time
+        date = time.localtime()
+        filename = "biomart_{}__{}_{}_{}.csv".format(self.dataset,
+            date.tm_year, date.tm_mon, date.tm_mday)
+        logger.info("Saving into {}".format(filename))
+        df.to_csv(filename, index=False)
 
 
