@@ -15,6 +15,7 @@
 #
 ##############################################################################
 "IEM class"
+import sys
 import collections
 from sequana import logger
 logger.name = __name__
@@ -101,7 +102,7 @@ class IEM():
 
         # if we are dealing with a section title, we can cleanup the 
         # line. A section must start with '[' and ends with ']' but
-        # there could be spaces and commads. 
+        # there could be spaces and commands. 
         if line.startswith('['):
             #[Header], ,, ,\n becomes [Header]
             line = line.strip(", ") # note the space AND comma
@@ -124,10 +125,10 @@ class IEM():
                 else:
                     data[current_section] += [line]
 
-        if "Header" not in data.keys():
+        if "Header" not in data.keys(): #pragma: no cover
             logger.warning("Input file must contain [Header]")
 
-        if "Data" not in data.keys():
+        if "Data" not in data.keys(): #pragma: no cover
             logger.warning("Input file must contain [Data]")
         self.data = data
 
@@ -140,7 +141,44 @@ class IEM():
     df = property(_get_df)
 
     def validate(self):
-        raise NotImplementedError
+        """This method checks whether the sample sheet is correctly formatted
+
+        Checks for:
+            * presence of ; at the end of lines indicated an edition with excel that
+              wrongly transformed the data into a pure CSV file
+            * inconsistent numbers of columns in the [DATA] section, which must be
+              CSV-like section
+        """
+        # could use logger, but simpler for now
+        # Note that this code is part of sequana_demultiplex
+        prefix = "ERROR  [sequana_pipelines.demultiplex.check_samplesheet]: "
+        try:
+            with open(self.filename, "r") as fp:
+               line = fp.readline()
+               cnt = 1
+               if line.rstrip().endswith(";") or line.rstrip().endswith(","): #pragma: no cover
+                   sys.exit(prefix + "Unexpected ; or , found at the end of line {} (and possibly others). Please use IEM  to format your SampleSheet. Try sequana_fix_samplesheet for extra ; or , ".format(cnt))
+
+               while line:
+                   line = fp.readline()
+                   cnt += 1
+                   if "[Data]" in line:
+                       line = fp.readline()
+                       cnt += 1
+                       if len(line.split(',')) < 2 or "Sample" not in line: #pragma:  no cover
+                           sys.exit(prefix + ": No header found after [DATA] section")
+                       line = fp.readline()
+                       cnt += 1
+                       nb_col = len(line.split(','))
+                       while line:
+                           if len(line.split(',')) != nb_col:
+                               sys.exit(prefix + "Different number of column in [DATA] section on line: "+str(cnt))
+                           line = fp.readline()
+                           cnt += 1
+        except Exception as e: #pragma: no cover
+            raise ValueError("type error: " + str(e))
+        return 0
+
 
     def _get_settings(self):
         data = {}
@@ -166,7 +204,7 @@ class IEM():
             return self.data['Name']
     name = property(_get_name)
 
-    def to_fasta(self, adapter_name):
+    def to_fasta(self, adapter_name=""):
         ar1 = self.settings['Adapter']
         try:ar2 = self.settings['AdapterRead2']
         except: ar2 =""
